#!/usr/bin/python

import sys
import subprocess
import re
import time
import syslog
import os
import datetime

rootwhitelist = ['/usr/bin/pbs_python', '/usr/sbin/smartd', '/usr/bin/ruby', '/usr/bin/nvidia-smi', '/sbin/mount.nfs']


def getUsers():
    try:
        with open(os.devnull, 'w') as devnull:
            proc = subprocess.check_output(["momctl", "-q", "users"], stderr=devnull)
    except:
        return []
    if proc:
        us = proc.split()[-1][:-1].split(',')
    else:
        us = []
    return us


def get_pstack_path(pid):
    start = "/proc"
    end = "stack"
    path = os.path.join(start, pid, end)
    return path


def check_nfs_rpc(pid):
    path = get_pstack_path(pid)
    try:
        with open(path, 'r') as file:
            for line in file:
                test_nfs = re.search("nfs", line)
                test_rpc = re.search("rpc", line)
                if test_nfs or test_rpc:
                    return True
    except:
        return False
    return False


def log(pid, lines):
    t = datetime.datetime.now()
    current_time = t.strftime("%H:%M:%S %d %b %Y")
    path = get_pstack_path(pid)
    try:
        with open("/var/tmp/dprocess.log", "a+") as file:
            with open(path, 'r') as stack_file:
	        file.write("{} {} ".format(current_time, lines))
                for l in stack_file:
                    file.write(l.rstrip())
            file.write("\n")
    except:
        return False
    return True


def getProcesses(activeu, on_off):
    pbs_mom = False
    healthcheck = False
    proc = subprocess.Popen(["ps", "ax", "-eo" "user:20,stat,pid,command"], stdout=subprocess.PIPE)
    lines = []
    for line in proc.stdout.readlines():
        splitted = line.split()
        lines.append(splitted)
        if splitted[3][0] == "/" and healthcheck:
            healthcheck = False
        if re.search('healthcheck', line) and splitted[3][0] == "/":
            healthcheck = True
            continue
        if ((re.search('D', splitted[1]) or re.search('Z', splitted[1])) and check_nfs_rpc(splitted[2])) and log(splitted[2], lines):
            ti = subprocess.Popen(["ps", "-o", "start", "-p", splitted[2]], stdout=subprocess.PIPE)
            for l in ti.stdout.readlines():
                continue
            l = l.rstrip()
            if healthcheck:
                print("Found something frozen under healthcheck,  on line \n " + line + "\nRunning iptables now\n")
                subprocess.call(["iptables", "-I", "OUTPUT", "-p", "tcp", "--dport", "2049", "-j", "REJECT", "--reject-with", "tcp-reset"])
                time.sleep(600)
                subprocess.call(["iptables", "-D", "OUTPUT", "-p", "tcp", "--dport", "2049", "-j", "REJECT", "--reject-with", "tcp-reset"])
                break
            if ((splitted[0] == "root") and (splitted[3].find("[") == -1) and (splitted[3] not in rootwhitelist)) or splitted[0] not in activeu:
                if on_off == "on":
                    print("Found sync under root or " + splitted[3] + " dfunct from " + l + ", running iptables \n" + line)
                    subprocess.call(["iptables", "-I", "OUTPUT", "-p", "tcp", "--dport", "2049", "-j", "REJECT", "--reject-with", "tcp-reset"])
                    time.sleep(600)
                    subprocess.call(["iptables", "-D", "OUTPUT", "-p", "tcp", "--dport", "2049", "-j", "REJECT", "--reject-with", "tcp-reset"])
                    break
	        else:
		    syslog.syslog("Found Z or D process from " + l + ", not running iptables \n" + line)
        


if __name__ == "__main__":
    iptables_on_off = sys.argv[1]
    users = getUsers()
    users.append("root")
    getProcesses(users, iptables_on_off)
